/*
 * Copyright ©2023 CJB-CNIT Team. All rights reserved
 */
package com.openhis.web.inhospitalnursestation.dto;

import java.util.List;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;

import lombok.Data;
import lombok.experimental.Accessors;

/**
 * 入出转管理页面初始化dto
 *
 * @author zwh
 * @date 2025-07-30
 */
@Data
@Accessors(chain = true)
public class ATDManageInitDto {

    /**
     * 住院状态
     */
    private List<ATDManageInitDto.encounterStatusOption> encounterStatusOptions;

    /**
     * 床位状态
     */
    private List<ATDManageInitDto.bedStatusOption> bedStatusOptions;

    /**
     * 患者病情
     */
    private List<ATDManageInitDto.priorityOption> priorityOptions;

    /**
     * 住院状态
     */
    @Data
    public static class encounterStatusOption {
        private Integer value;
        private String label;

        public encounterStatusOption(Integer value, String label) {
            this.value = value;
            this.label = label;
        }
    }

    /**
     * 床位状态
     */
    @Data
    public static class bedStatusOption {
        private Integer value;
        private String label;

        public bedStatusOption(Integer value, String label) {
            this.value = value;
            this.label = label;
        }
    }

    /**
     * 患者病情
     */
    @Data
    public static class priorityOption {
        private Integer value;
        private String label;

        public priorityOption(Integer value, String label) {
            this.value = value;
            this.label = label;
        }
    }
}
